@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '0.983';
+$PAR::VERSION = '1.000';
 
 use 5.006;
 use strict;
@@ -33,10 +33,6 @@ use PAR::SetupTemp;
 
 PAR - Perl Archive Toolkit
 
-=head1 VERSION
-
-This document describes release 0.983 of PAR, released September 12, 2008.
-
 =head1 SYNOPSIS
 
 (If you want to make an executable that contains all module, scripts and
@@ -135,6 +131,13 @@ If you specify the C<install =E<gt> 1> option in the C<use PAR>
 line above, the distribution containing C<Module> will be permanently
 installed on your system. (C<use PAR { repository =E<gt> 'http://foo/bar', install =E<gt> 1 };>)
 
+Furthermore, there is an C<upgrade =E<gt> 1> option that checks for upgrades
+in the repository in addition to installing. Please note that an upgraded
+version of a module is only loaded on the next run of your application.
+
+Adding the C<dependencies =E<gt> 1> option will enable PAR::Repository::Client's
+static dependency resolution (PAR::Repository::Client 0.23 and up).
+
 Finally, you can combine the C<run> and C<repository>
 options to run an application directly from a repository! (And you can add
 the C<install> option, too.)
@@ -278,7 +281,15 @@ A par repository (exclusive of file)
 
 =item fallback
 
-Search the system @INC before the par.
+Search the system C<@INC> before the par.
+
+Off by default for loading F<.par> files via C<file => ...>.
+On by default for PAR repositories.
+
+To prefer loading modules from a repository over the locally
+installed modules, you can load the repository as follows:
+
+  use PAR { repository => 'http://foo/bar/', fallback => 0 };
 
 =item run
 
@@ -298,16 +309,20 @@ shared object file..."
 
 =cut
 
-use vars qw(@PAR_INC);              # explicitly stated PAR library files (prefered)
+use vars qw(@PAR_INC);              # explicitly stated PAR library files (preferred)
 use vars qw(@PAR_INC_LAST);         # explicitly stated PAR library files (fallback)
 use vars qw(%PAR_INC);              # sets {$par}{$file} for require'd modules
 use vars qw(@LibCache %LibCache);   # I really miss pseudohash.
 use vars qw($LastAccessedPAR $LastTempFile);
 use vars qw(@RepositoryObjects);    # If we have PAR::Repository::Client support, we
                                     # put the ::Client objects in here.
+use vars qw(@PriorityRepositoryObjects); # repositories which are preferred over local stuff
+use vars qw(@UpgradeRepositoryObjects);  # If we have PAR::Repository::Client's in upgrade mode
+                                         # put the ::Client objects in here *as well*.
 use vars qw(%FileCache);            # The Zip-file file-name-cache
                                     # Layout:
                                     # $FileCache{$ZipObj}{$FileName} = $Member
+use vars qw(%ArchivesExtracted);    # Associates archive-zip-object => full extraction path
 
 my $ver  = $Config{version};
 my $arch = $Config{archname};
@@ -333,6 +348,10 @@ sub import {
 
     my @args = @_;
     
+    # Insert PAR hook in @INC.
+    unshift @INC, \&find_par   unless grep { $_ eq \&find_par }      @INC;
+    push @INC, \&find_par_last unless grep { $_ eq \&find_par_last } @INC;
+
     # process args to use PAR 'foo.par', { opts }, ...;
     foreach my $par (@args) {
         if (ref($par) eq 'HASH') {
@@ -355,10 +374,6 @@ sub import {
     return if $PAR::__import;
     local $PAR::__import = 1;
 
-    # Insert PAR hook in @INC.
-    unshift @INC, \&find_par   unless grep { $_ eq \&find_par }      @INC;
-    push @INC, \&find_par_last unless grep { $_ eq \&find_par_last } @INC;
-
     require PAR::Heavy;
     PAR::Heavy::_init_dynaloader();
 
@@ -370,6 +385,11 @@ sub import {
         push @PAR_INC, unpar($progname, undef, undef, 1);
 
         _extract_inc($progname) unless $ENV{PAR_CLEAN};
+        if ($LibCache{$progname}) {
+          # XXX bad: this us just a good guess
+          require File::Spec;
+          $ArchivesExtracted{$progname} = File::Spec->catdir($ENV{PAR_TEMP}, 'inc');
+        }
 
         my $zip = $LibCache{$progname};
         my $member = _first_member( $zip,
@@ -502,9 +522,17 @@ sub _import_repository {
     my $url = $opt->{repository};
 
     eval "require PAR::Repository::Client; 1;";
-    if ($@ or not PAR::Repository::Client->VERSION >= 0.04) {
+    if ($@ or not eval PAR::Repository::Client->VERSION >= 0.04) {
         croak "In order to use the 'use PAR { repository => 'url' };' syntax, you need to install the PAR::Repository::Client module (version 0.04 or later) from CPAN. This module does not seem to be installed as indicated by the following error message: $@";
     }
+    
+    if ($opt->{upgrade} and not eval PAR::Repository::Client->VERSION >= 0.22) {
+        croak "In order to use the 'upgrade' option, you need to install the PAR::Repository::Client module (version 0.22 or later) from CPAN";
+    }
+
+    if ($opt->{dependencies} and not eval PAR::Repository::Client->VERSION >= 0.23) {
+        croak "In order to use the 'dependencies' option, you need to install the PAR::Repository::Client module (version 0.23 or later) from CPAN";
+    }
 
     my $obj;
 
@@ -514,12 +542,21 @@ sub _import_repository {
     }
     else {
         $obj = PAR::Repository::Client->new(
-            uri => $url,
-            auto_install => $opt->{install},
+            uri                 => $url,
+            auto_install        => $opt->{install},
+            auto_upgrade        => $opt->{upgrade},
+            static_dependencies => $opt->{dependencies},
         );
     }
 
-    push @RepositoryObjects, $obj;
+    if (exists($opt->{fallback}) and not $opt->{fallback}) {
+        push @PriorityRepositoryObjects, $obj; # repository beats local stuff
+    } else {
+        push @RepositoryObjects, $obj; # local stuff beats repository
+    }
+    # these are tracked separately so we can check for upgrades early
+    push @UpgradeRepositoryObjects, $obj if $opt->{upgrade};
+
     return $obj;
 }
 
@@ -536,6 +573,26 @@ sub _first_member {
     return;
 }
 
+# Given an Archive::Zip object, this finds the first 
+# Archive::Zip member whose file name matches the
+# regular expression
+sub _first_member_matching {
+    my $zip = shift;
+    my $regex = shift;
+
+    my $cache = $FileCache{$zip};
+    $cache = $FileCache{$zip} = _make_file_cache($zip) if not $cache;
+
+    foreach my $name (keys %$cache) {
+      if ($name =~ $regex) {
+        return $cache->{$name};
+      }
+    }
+
+    return();
+}
+
+
 sub _run_member_from_par {
     my $member = shift;
     my $clear_stack = shift;
@@ -583,6 +640,34 @@ sub _run_member {
     }
 }
 
+sub _run_external_file {
+    my $filename = shift;
+    my $clear_stack = shift;
+    require 5.008;
+    open my $ffh, '<', $filename
+      or die "Can't open perl script \"$filename\": $!";
+
+    my $clearstack = '';
+    if (defined &Internals::PAR::CLEARSTACK and $clear_stack) {
+        $clear_stack = "Internals::PAR::CLEARSTACK();\n";
+    }
+    my $string = "package main; shift \@INC;\n$clearstack#line 1 \"$filename\"\n"
+                 . do { local $/ = undef; <$ffh> };
+    close $ffh;
+
+    open my $fh, '<', \$string
+      or die "Can't open file handle to string: $!";
+
+    unshift @INC, sub { $fh };
+
+    $ENV{PAR_0} = $filename; # for Pod::Usage
+    { do 'main';
+      CORE::exit($1) if ($@ =~/^_TK_EXIT_\((\d+)\)/);
+      die $@ if $@;
+      exit;
+    }
+}
+
 # extract the contents of a .par (or .exe) or any
 # Archive::Zip handle to the PAR_TEMP/inc directory.
 # returns that directory.
@@ -645,6 +730,8 @@ sub _extract_inc {
         }
         
         rmdir("$inc.lock");
+
+        $ArchivesExtracted{$is_handle ? $file_or_azip_handle->fileName() : $file_or_azip_handle} = $inc;
     }
 
     # add the freshly extracted directories to @INC,
@@ -663,7 +750,58 @@ sub _extract_inc {
 # This is the hook placed in @INC for loading PAR's
 # before any other stuff in @INC
 sub find_par {
-    return _find_par_internals(\@PAR_INC, @_);
+    my @args = @_;
+
+    # if there are repositories in upgrade mode, check them
+    # first. If so, this is expensive, of course!
+    if (@UpgradeRepositoryObjects) {
+        my $module = $args[1];
+        $module =~ s/\.pm$//;
+        $module =~ s/\//::/g;
+        foreach my $client (@UpgradeRepositoryObjects) {
+            my $local_file = $client->upgrade_module($module);
+
+            # break the require if upgrade_module has been required already
+            # to avoid infinite recursion
+            if (exists $INC{$args[1]}) {
+                # Oh dear. Check for the possible return values of the INC sub hooks in
+                # perldoc -f require before trying to understand this.
+                # Then, realize that if you pass undef for the file handle, perl (5.8.9)
+                # does NOT use the subroutine. Thus the hacky GLOB ref.
+                my $line = 1;
+                no warnings;
+                return (\*I_AM_NOT_HERE, sub {$line ? ($_="1;",$line=0,return(1)) : ($_="",return(0))});
+            }
+
+            # Note: This is likely not necessary as the module has been installed
+            # into the system by upgrade_module if it was available at all.
+            # If it was already loaded, this will not be reached (see return right above).
+            # If it could not be loaded from the system and neither found in the repository,
+            # we simply want to have the normal error message, too!
+            #
+            #if ($local_file) {
+            #    # XXX load with fallback - is that right?
+            #    return _find_par_internals([$PAR_INC_LAST[-1]], @args);
+            #}
+        }
+    }
+    my $rv = _find_par_internals(\@PAR_INC, @args);
+
+    return $rv if defined $rv or not @PriorityRepositoryObjects;
+
+    # the repositories that are prefered over locally installed modules
+    my $module = $args[1];
+    $module =~ s/\.pm$//;
+    $module =~ s/\//::/g;
+    foreach my $client (@PriorityRepositoryObjects) {
+        my $local_file = $client->get_module($module, 0); # 1 == fallback
+        if ($local_file) {
+            # Not loaded as fallback (cf. PRIORITY) thus look at PAR_INC
+            # instead of PAR_INC_LAST
+            return _find_par_internals([$PAR_INC[-1]], @args);
+        }
+    }
+    return();
 }
 
 # This is the hook placed in @INC for loading PAR's
@@ -683,8 +821,9 @@ sub find_par_last {
     $module =~ s/\.pm$//;
     $module =~ s/\//::/g;
     foreach my $client (@RepositoryObjects) {
-        my $local_file = $client->get_module($module, 1);
+        my $local_file = $client->get_module($module, 1); # 1 == fallback
         if ($local_file) {
+            # Loaded as fallback thus look at PAR_INC_LAST
             return _find_par_internals([$PAR_INC_LAST[-1]], @args);
         }
     }
@@ -692,14 +831,6 @@ sub find_par_last {
 }
 
 
-# This is a conjunction of the early find_par and the late
-# find_par_last. It's called by PAR::Heavy for Dynaloader stuff.
-sub _find_par_any {
-    return _find_par_internals([@PAR_INC, @PAR_INC_LAST], @_);
-}
-
-
-
 # This routine implements loading modules from PARs
 # both for loading PARs preferably or as fallback.
 # To distinguish the cases, the first parameter should
@@ -710,7 +841,7 @@ sub _find_par_internals {
     my $scheme;
     foreach (@$INC_ARY ? @$INC_ARY : @INC) {
         my $path = $_;
-        if ($[ < 5.008001) {
+        if ($] < 5.008001) {
             # reassemble from "perl -Ischeme://path" autosplitting
             $path = "$scheme:$path" if !@$INC_ARY
                 and $path and $path =~ m!//!
@@ -1024,7 +1155,19 @@ sub _cached_member_named {
 }
 
 
-
+# Attempt to clean up the temporary directory if
+# --> We're running in clean mode
+# --> It's defined
+# --> It's an existing directory
+# --> It's empty
+END {
+  if (exists $ENV{PAR_CLEAN} and $ENV{PAR_CLEAN}
+      and exists $ENV{PAR_TEMP} and defined $ENV{PAR_TEMP} and -d $ENV{PAR_TEMP}
+  ) {
+    local($!); # paranoid: ignore potential errors without clobbering a global variable!
+    rmdir($ENV{PAR_TEMP});
+  }
+}
 
 1;
 
@@ -1049,6 +1192,9 @@ L<Archive::Zip>, L<perlfunc/require>
 
 L<ex::lib::zip>, L<Acme::use::strict::with::pride>
 
+Steffen Mueller has detailed slides on using PAR for application
+deployment at L<http://steffen-mueller.net/talks/appdeployment/>.
+
 PAR supports the L<prefork> module. It declares various run-time
 dependencies so you can use the L<prefork> module to get streamlined
 processes in a forking environment.
@@ -1074,6 +1220,8 @@ have sent helpful patches, ideas or comments.
 
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
+Steffen Mueller E<lt>smueller@cpan.orgE<gt>
+
 L<http://par.perl.org/> is the official PAR website.  You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
@@ -1084,8 +1232,9 @@ preferred.
 
 =head1 COPYRIGHT
 
-Copyright 2002-2008 by Audrey Tang
+Copyright 2002-2010 by Audrey Tang
 E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2005-2010 by Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
